import { actionNewOrder } from "actions";
import { Link } from "react-router-dom";
import { actionCartDelete } from "reducers";

const { List, Divider, ListItem, Typography, Button, TableRow, TableBody, Table, TableCell } = require("@mui/material");
const { useSelector, useDispatch } = require("react-redux");
const { DrawerCartItem } = require("./DrawerCartItem");
const { DrawerRight } = require("./DrawerRight");

export const DrawerCart = ({ isOpen = false, onClose = null } = {}) => {
    const cart = useSelector((state) => state.cart || {});
    const sum = Object.entries(cart).reduce((prev, [_id, order]) => prev + order.count * order.good.price, 0);
    const dispatch = useDispatch();

    return (
        <DrawerRight className="DrawerCart" open={isOpen} onClose={() => onClose()}>
            <Table sx={{ width: 400 }}>
                <TableBody>
                    <TableRow>
                        <TableCell colSpan={5}>
                            <Typography variant="h5" textAlign="center">
                                Корзина
                            </Typography>
                        </TableCell>
                    </TableRow>

                    {Object.entries(cart).map(([_id, order]) => (
                        <DrawerCartItem order={order} onDeleteClick={(good) => dispatch(actionCartDelete(good))} />
                    ))}

                    <TableRow>
                        <TableCell colSpan={5}>
                            <Typography variant="body1">
                                <b>Весего:</b>&nbsp;
                                {sum}
                            </Typography>
                        </TableCell>
                    </TableRow>

                    {!!Object.keys(cart).length && (
                        <TableRow>
                            <TableCell align="right" colSpan={5}>
                                <Button
                                    variant="text"
                                    onClick={() => {
                                        dispatch(actionNewOrder([...Object.values(cart)]));
                                        onClose();
                                    }}
                                >
                                    Подтвердить
                                </Button>
                            </TableCell>
                        </TableRow>
                    )}
                </TableBody>
            </Table>
        </DrawerRight>
    );
};
