import { useEffect, useState } from "react";
import { arrayMoveImmutable } from "array-move";
import { DropZone } from "./DropZone";
import { SortableList } from "./SortableList";
import { SortableItem } from "./SortableItem";
import { backendURL } from "../helpers";
import { Box, Button, Typography } from "@mui/material";

export const EntityEditor = ({ entity = { images: [] }, onSave, onFileDrop, uploadFiles, onImagesSave }) => {
    const [state, setState] = useState(entity);

    useEffect(() => {
        setState(entity);
    }, [entity]);

    useEffect(() => {
        if (uploadFiles?.status === "FULFILLED") {
            setState({ ...state, images: [...state.images, ...uploadFiles?.payload] });
        }
    }, [uploadFiles]);

    useEffect(() => {
        onImagesSave && onImagesSave(state.images?.filter((img) => img?._id && img?.url));
    }, [state.images]);

    const onSortEnd = ({ oldIndex, newIndex }) => {
        setState({ ...state, images: arrayMoveImmutable(state.images, oldIndex, newIndex) });
    };
    const onItemRemove = (toRemoveId) => {
        console.log(toRemoveId);
        setState({ ...state, images: [...state.images.filter((el) => el?._id !== toRemoveId)] });
    };

    return (
        <Box className="EntityEditor">
            <DropZone onFileDrop={(files) => onFileDrop(files)}>
                <Typography>Drop images here!</Typography>
            </DropZone>
            <SortableList pressDelay={200} onSortEnd={onSortEnd} axis="xy" className="SortableContainer">
                {state.images?.map(
                    (image, index) =>
                        !!image?._id &&
                        !!image?.url && (
                            <SortableItem key={`item-${image._id}`} index={index}>
                                <img className="DropZoneImage" src={`${backendURL}/${image.url}`} />
                                <div className="closeButton" onClick={() => onItemRemove(image._id)}>
                                    X
                                </div>
                            </SortableItem>
                        )
                )}
            </SortableList>
            {!!onSave && (
                <Button
                    onClick={() => {
                        console.log({ _id: entity._id, images: state.images });
                        onSave(entity._id, state.images);
                    }}
                >
                    Save
                </Button>
            )}
        </Box>
    );
};
