import { backendURL } from "helpers";
import { connect, useSelector } from "react-redux";
import { actionCartAdd } from "reducers";
import { Link } from "react-router-dom";
import CBuyButton from "components/BuyButton";
import defaultGoodImage from "images/defaultGoodImage.png";
import { Button, Card, CardActionArea, CardActions, CardContent, CardMedia, Typography } from "@mui/material";

export const GoodCard = ({ good: { name, images, price, _id }, handleOnClick, buyButton = true } = {}) => {
    const token = useSelector((state) => state.auth?.token || null);
    return (
        <Card className="GoodCard">
            <Link className="Link" to={`/good/${_id}`}>
                <CardActionArea>
                    <CardMedia
                        component="img"
                        height="140"
                        image={
                            images && images[0]?.url
                                ? `${backendURL}/${images ? images[0]?.url : ""}`
                                : defaultGoodImage
                        }
                    />

                    <CardContent sx={{ textAlign: "left" }}>
                        <Typography gutterBottom variant="h6" component="div" noWrap>
                            {name}
                        </Typography>
                        <Typography variant="body2" color="text.secondary">
                            <strong>Цена: </strong> {price}
                        </Typography>
                    </CardContent>
                </CardActionArea>
            </Link>
            {token && buyButton ? (
                <CardActions>
                    <Button size="small" color="primary" sx={{ ml: "auto" }}>
                        <CBuyButton good={{ name, images, price, _id }} key={_id} />
                    </Button>
                </CardActions>
            ) : (
                false
            )}
        </Card>
    );
};
export const CGoodCard = connect(null, {
    handleOnClick: (good) => actionCartAdd(good),
})(GoodCard);
