import { useEffect, useState, useRef } from "react";
import { connect, useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { actionGoodsFind } from "actions";
import { actionPromiseClear, store } from "reducers";
import { CSearchResults } from ".";

import { Box, TextField, Button, Container, Stack } from "@mui/material";

import {} from "react-loader-spinner";

// import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
// import { faSearch } from "@fortawesome/free-solid-svg-icons";

export const SearchBar = ({
    onSearch,
    onSearchButtonClick,
    render = null,
    renderParams = {},
    searchLink = "/search/",
} = {}) => {
    const ref = useRef();
    const [inputValue, setInputValue] = useState("");
    const [isChildrenOpen, setIsChildrenOpen] = useState(false);
    const [inputTimeout, setInputTimeout] = useState(null);
    const R = render;

    useEffect(() => {
        if (inputTimeout) {
            clearTimeout(inputTimeout);
        }

        const checkClickOutsideHeaderSearchBar = (e) => {
            if (ref.current && !ref.current.contains(e.target)) {
                setIsChildrenOpen(false);
            } else {
                inputValue.length && setIsChildrenOpen(true);
            }
        };

        inputValue && onSearch(inputValue);
        setIsChildrenOpen(!!inputValue?.length);
        document.addEventListener("mousedown", checkClickOutsideHeaderSearchBar);
        return () => {
            document.removeEventListener("mousedown", checkClickOutsideHeaderSearchBar);
        };
    }, [inputValue]);

    return (
        <Box className={`SearchBar ${!isChildrenOpen && "hide"}`} ref={ref}>
            <Stack direction="row" justifyContent="center" alignItems="center">
                <TextField
                    variant="standard"
                    value={inputValue}
                    placeholder="Поиск"
                    onChange={(e) => setInputValue(e.target.value)}
                    sx={{ width: "40%" }}
                    size="small"
                />

                {!!inputValue ? (
                    <Link
                        className="Link"
                        to={`${searchLink}${inputValue}`}
                        onClick={() => {
                            setInputValue("");
                            onSearchButtonClick();
                        }}
                    >
                        <Button variant="text" sx={{ alignSelf: "flex-end", color: "white" }}>
                            Поиск
                        </Button>
                    </Link>
                ) : (
                    <Button variant="text" sx={{ alignSelf: "flex-end", color: "white" }}>
                        Поиск
                    </Button>
                )}
            </Stack>
            <Stack direction="row" justifyContent="center">
                {isChildrenOpen && (
                    <R
                        onItemClick={() => {
                            setInputValue("");
                            onSearchButtonClick();
                        }}
                        {...renderParams}
                    />
                )}
            </Stack>
        </Box>
    );
};

export const CSearchBar = connect(null, {
    onSearch: (text) => actionGoodsFind({ text, limit: 5 }),
    onSearchButtonClick: () => actionPromiseClear("goodsFind"),
})(SearchBar);
