import { connect } from "react-redux";
import React, { useState, useEffect } from "react";
import { Oval } from "react-loader-spinner";
import Select from "react-select";
import { actionCategoryUpdate } from "actions/actionCategoryUpdate";
import { actionPromise, actionPromiseClear, store } from "reducers";
import { SuccessIcon } from "components/SuccessIcon";
import { FailedIcon } from "components/FailedIcon";
import { Box, Button, InputLabel, Stack, TextField, Typography } from "@mui/material";

const CategoryForm = ({
    serverErrors,
    onSaveClick,
    onSave,
    onClose,
    promiseStatus,
    goodsField = false,
    catList: initialCatList = [],
    goodList = [],
    category = {},
} = {}) => {
    const defaultErrorSchema = {
        name: [],
        parent: [],
        subCatergories: [],
        goods: [],
    };
    const [inputName, setInputName] = useState("");
    const [inputSubCategories, setInputSubCategories] = useState([]);
    const [inputGoods, setInputGoods] = useState([]);
    const [inputParent, setInputParent] = useState({});
    const [isValid, setIsValid] = useState(false);
    const [subCatList, setSubCatList] = useState([]);
    const [parentList, setParentList] = useState([]);
    const [errorSchema, setErrorSchema] = useState(defaultErrorSchema);

    useEffect(() => {
        setInputParent(category?.parent || null);
        setInputName(category?.name || "");
        setInputGoods(category?.goods || []);
        setInputSubCategories(category?.subCategories || []);
    }, [category]);

    useEffect(() => {
        let parentList = initialCatList.filter(
            ({ _id }) =>
                !category?.subCatergories?.find((subCat) => _id === subCat._id) &&
                _id !== category?._id &&
                !inputSubCategories?.find((subCat) => _id === subCat._id)
        );
        parentList = [...[{ _id: null, name: "null" }], ...parentList];

        setParentList(parentList);
    }, [inputSubCategories]);

    useEffect(() => {
        let subCatList = initialCatList.filter(
            ({ _id }) => _id !== category?.parent?._id && _id !== category?._id && inputParent?._id !== _id
        );
        setSubCatList(subCatList);
    }, [inputParent]);

    useEffect(() => {
        return () => {
            onClose && onClose();
        };
    }, []);

    // useEffect(() => {
    //     let tempErrorsSchema = { ...defaultErrorSchema };
    //     const validationSchema = {
    //         type: "object",
    //         properties: {
    //             login: {
    //                 type: "string",
    //                 minLength: 3,
    //                 maxLength: 15,
    //             },
    //             password1: {
    //                 type: "string",
    //                 minLength: 3,
    //                 maxLength: 12,
    //             },
    //             password2: {
    //                 type: "string",
    //                 minLength: 3,
    //                 maxLength: 12,
    //             },
    //         },
    //         required: ["login", "password1", "password2"],
    //         additionalProperties: false,
    //     };

    //     let errors = formValidate(validationSchema, {
    //         login: inputUsername,
    //         password1: inputPassword1,
    //         password2: inputPassword2,
    //     });

    //     tempErrorsSchema = { ...tempErrorsSchema, ...errors };

    //     setErrorSchema(tempErrorsSchema);
    // }, [inputUsername, inputPassword1, inputPassword2]);

    useEffect(() => {
        let valid = true;
        for (let errorList of Object.values(errorSchema)) {
            errorList?.length > 0 && (valid = false);
        }

        setIsValid(valid);
    }, [errorSchema]);

    return (
        <Box className="CategoryForm">
            {(serverErrors || []).map((error) => (
                <Typography color="red">{error?.message}</Typography>
            ))}

            <Box>
                <InputLabel>Название</InputLabel>
                <TextField
                    className={`form-control ${errorSchema["name"].length > 0 && `is-invalid`}`}
                    id="name"
                    name="name"
                    placeholder="Название"
                    variant="standard"
                    value={inputName}
                    onChange={(e) => setInputName(e.target.value)}
                />
                {(errorSchema["name"] || []).map((error) => (
                    <Typography color="red">{error}</Typography>
                ))}
            </Box>
            <Box sx={{ mt: 3 }}>
                <InputLabel className="form-label">Родительская категория</InputLabel>

                <Select
                    value={{ value: inputParent?._id || null, label: inputParent?.name || "null" }}
                    onChange={(e) => setInputParent({ _id: e.value, name: e.label })}
                    options={parentList.map(({ _id, name }) => ({ value: _id, label: name }))}
                />
                {(errorSchema["parent"] || []).map((error) => (
                    <Typography color="red">{error}</Typography>
                ))}
            </Box>

            <Box sx={{ mt: 3 }}>
                <InputLabel className="form-label">Подкатегории</InputLabel>
                <Select
                    value={inputSubCategories?.map(({ _id, name }) => ({ value: _id, label: name }))}
                    closeMenuOnSelect={false}
                    onChange={(e) => setInputSubCategories(e.map(({ value, label }) => ({ _id: value, name: label })))}
                    options={subCatList?.map(({ _id, name }) => ({ value: _id, label: name }))}
                    isMulti={true}
                />

                {(errorSchema["subCatergories"] || []).map((error) => (
                    <Typography color="red">{error}</Typography>
                ))}
            </Box>
            {goodsField && (
                <Box sx={{ mt: 3 }}>
                    <InputLabel className="form-label">Товары</InputLabel>
                    <Select
                        value={inputGoods?.map(({ _id, name }) => ({ value: _id, label: name }))}
                        closeMenuOnSelect={false}
                        onChange={(e) => setInputGoods(e.map(({ value, label }) => ({ _id: value, name: label })))}
                        options={goodList?.map(({ _id, name }) => ({ value: _id, label: name }))}
                        isMulti={true}
                    />

                    {(errorSchema["subCatergories"] || []).map((error) => (
                        <Typography color="red">{error}</Typography>
                    ))}
                </Box>
            )}

            <Stack direction="row" className="confirmButton" sx={{ mt: 3 }}>
                <Button
                    disabled={!isValid}
                    onClick={() => {
                        let categoryToSave = {};
                        category?._id && (categoryToSave._id = category?._id);
                        categoryToSave.name = inputName;
                        inputGoods && (categoryToSave.goods = inputGoods);
                        inputParent && (categoryToSave.parent = inputParent);

                        categoryToSave.subCategories = inputSubCategories;
                        onSaveClick && onSaveClick();
                        onSave(categoryToSave);
                    }}
                >
                    Сохранить
                </Button>
                <Box className="status">
                    {!!promiseStatus && promiseStatus === "PENDING" ? (
                        <Oval color="#000" height={20} width={20} />
                    ) : promiseStatus === "FULFILLED" ? (
                        <SuccessIcon color="#0F0" fontSize={30} />
                    ) : promiseStatus === "REJECTED" ? (
                        <FailedIcon color="#F00" fontSize={30} />
                    ) : (
                        ""
                    )}
                </Box>
            </Stack>
        </Box>
    );
};

// const CRegisterForm = connect((state) => ({ serverErrors: state.promise?.register?.error || [] }), {
//     onRegister: (login, password) => actionRegister(login, password),
// })(RegisterForm);

export const CCategoryForm = connect(
    (state) => ({
        catList: state.promise.catAll?.payload || [],
        promiseStatus: state.promise.categoryUpsert?.status || null,
        goodList: state.promise.goodsAll?.payload || [],
    }),
    {
        onSave: (cat) => actionCategoryUpdate(cat),
        onClose: () => actionPromiseClear("categoryUpsert"),
    }
)(CategoryForm);
