import { DropZone } from "components/DropZone";
import { connect } from "react-redux";
import { CProfileImage } from "components/ProfileImage";
import { actionUpdateAvatar } from "actions";
import { useState, useEffect } from "react";
import { actionUserUpdate } from "actions/actionUserUpdate";
import { ProfileFormInput } from "./ProfileFormInput";
import { FaEdit } from "react-icons/fa";
import {
    Box,
    IconButton,
    Paper,
    Stack,
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableRow,
    Typography,
} from "@mui/material";

const CDropZone = connect(null, { onFileDrop: (acceptedFiles) => actionUpdateAvatar(acceptedFiles[0]) })(DropZone);

export const ProfileForm = ({ profile, onProfileSave }) => {
    const { nick, createdAt, login } = profile || {};
    const [isNickChanging, setIsNickChanging] = useState(false);

    const [isLetterShown, setIsLetterShown] = useState(false);

    return (
        <Paper className="ProfileForm">
            <Stack direction="row" spacing={3}>
                <Box sx={{ maxWidth: "800px" }}>
                    <CDropZone>
                        <Box
                            className="profileImageWrapper"
                            onMouseEnter={() => setIsLetterShown(true)}
                            onMouseLeave={() => setIsLetterShown(false)}
                        >
                            <CProfileImage />
                            <div className={`letter ${isLetterShown && "show"}`}>Drop file or click to update</div>
                        </Box>
                    </CDropZone>
                </Box>
                <TableContainer sx={{ flexGrow: 1 }}>
                    <Table>
                        <TableBody>
                            <TableRow>
                                <TableCell>Имя пользоватиля:</TableCell>
                                <TableCell>
                                    {isNickChanging ? (
                                        <ProfileFormInput
                                            value={nick}
                                            onSave={(newNick) => {
                                                newNick.length && newNick !== nick && onProfileSave({ nick: newNick });
                                                setIsNickChanging(false);
                                            }}
                                        />
                                    ) : (
                                        nick
                                    )}
                                </TableCell>
                                <TableCell className="editButtonWrapper">
                                    {!isNickChanging && (
                                        <IconButton onClick={() => setIsNickChanging(true)}>
                                            <FaEdit />
                                        </IconButton>
                                    )}
                                </TableCell>
                            </TableRow>
                            <TableRow>
                                <TableCell>Дата регистрации:</TableCell>
                                <TableCell>
                                    {!!createdAt && (
                                        <Typography variant="body1">
                                            {`00${new Date(+createdAt).getDay()}`.slice(-2)}/
                                            {`00${new Date(+createdAt).getMonth()}`.slice(-2)}/
                                            {`0000${new Date(+createdAt).getFullYear()}`.slice(-4)}
                                        </Typography>
                                    )}
                                </TableCell>
                                <TableCell></TableCell>
                            </TableRow>
                            <TableRow>
                                <TableCell>Логин:</TableCell>
                                <TableCell>{login}</TableCell>
                                <TableCell></TableCell>
                            </TableRow>
                        </TableBody>
                    </Table>
                </TableContainer>
            </Stack>
        </Paper>
    );
};

export const CProfileForm = connect((state) => ({ profile: state.promise?.aboutMe?.payload }), {
    onProfileSave: (profile = {}) => actionUserUpdate(profile),
})(ProfileForm);
