import { Component, useState, useEffect } from "react";
import { connect } from "react-redux";
import { actionRegister } from "actions";
import { formValidate } from "helpers";

// class RegisterForm extends Component {
//     state = {
//         errors: { inputUsername: [], inputPassword1: [], inputPassword2: [] },
//         inputUsername: "",
//         inputPassword1: "",
//         inputPassword2: "",
//         isValid: false,
//     };
//     setErrors = (value) => {
//         this.setState({ errors: value });
//     };
//     setInputUsername = (value) =>
//         this.setState({ inputUsername: value }, () => {
//             this.validate();
//         });
//     setInputPassword1 = (value) =>
//         this.setState({ inputPassword1: value }, () => {
//             this.validate();
//         });
//     setInputPassword2 = (value) =>
//         this.setState({ inputPassword2: value }, () => {
//             this.validate();
//         });
//     setIsValid = (value) => this.setState({ isValid: value });

//     handleOnRegister = () => {
//         const { inputUsername, inputPassword1 } = this.state;
//         const { onRegister } = this.props;
//         onRegister(inputUsername, inputPassword1);
//     };

//     handleOnInputUsernameChange = (value) => {
//         const { setInputUsername } = this;
//         setInputUsername(value);
//     };
//     handleOnInputPassword1Change = (value) => {
//         const { setInputPassword1 } = this;
//         setInputPassword1(value);
//     };

//     handleOnInputPassword2Change = (value) => {
//         const { setInputPassword2 } = this;
//         setInputPassword2(value);
//     };

//     validate = () => {
//         const { inputUsername, inputPassword1, inputPassword2, errors } = this.state;
//         const { setErrors, setIsValid } = this;
//         let newErrors = { ...errors };
//         setIsValid(true);
//         if (inputUsername.length < 4) {
//             newErrors.inputUsername = [...new Set([...newErrors["inputUsername"], ...["Никнейм слишком короткий"]])];
//         } else {
//             newErrors.inputUsername = newErrors["inputUsername"].filter((el) => el !== "Никнейм слишком короткий");
//         }

//         if (inputPassword1 && inputPassword1.length < 5) {
//             newErrors.inputPassword1 = [...new Set([...newErrors["inputPassword1"], ...["Пароль слишком короткий"]])];
//         } else {
//             newErrors.inputPassword1 = newErrors["inputPassword1"]?.filter((el) => el !== "Пароль слишком короткий");
//         }

//         if (inputPassword1 && inputPassword2 && inputPassword1 !== inputPassword2) {
//             newErrors.inputPassword2 = [...new Set([...newErrors["inputPassword2"], ...["Пароли не совпадают"]])];
//         } else {
//             newErrors.inputPassword2 = newErrors["inputPassword2"]?.filter((el) => el !== "Пароли не совпадают");
//         }

//         for (let array of Object.values(newErrors)) {
//             array.length > 0 && setIsValid(false);
//         }

//         setErrors({ ...errors, ...newErrors });
//     };

//     render() {
//         let { serverError } = this.props;
//         let serverErrors = serverError?.message?.match(/{"message":"(.*)",/)[1] || "";
//         console.log(serverErrors);
//         const { isValid, inputUsername, inputPassword1, inputPassword2, errors } = this.state;

//         const {
//             handleOnInputUsernameChange,
//             handleOnInputPassword1Change,
//             handleOnInputPassword2Change,
//             handleOnRegister,
//         } = this;

//         return (

//         );
//     }
// }

const RegisterForm = ({ serverErrors, onRegister }) => {
    const defaultErrorSchema = {
        login: [],
        password1: [],
        password2: [],
    };
    const [inputUsername, setInputUsername] = useState("");
    const [inputPassword1, setInputPassword1] = useState("");
    const [inputPassword2, setInputPassword2] = useState("");
    const [isValid, setIsValid] = useState(false);
    const [errorSchema, setErrorSchema] = useState(defaultErrorSchema);

    useEffect(() => {
        let tempErrorsSchema = { ...defaultErrorSchema };
        const validationSchema = {
            type: "object",
            properties: {
                login: {
                    type: "string",
                    minLength: 3,
                    maxLength: 15,
                },
                password1: {
                    type: "string",
                    minLength: 3,
                    maxLength: 12,
                },
                password2: {
                    type: "string",
                    minLength: 3,
                    maxLength: 12,
                },
            },
            required: ["login", "password1", "password2"],
            additionalProperties: false,
        };

        let errors = formValidate(validationSchema, {
            login: inputUsername,
            password1: inputPassword1,
            password2: inputPassword2,
        });

        tempErrorsSchema = { ...tempErrorsSchema, ...errors };

        if (inputPassword1 && inputPassword2 && inputPassword1 !== inputPassword2) {
            tempErrorsSchema.password2.push("Passwords should match");
        }

        setErrorSchema(tempErrorsSchema);
    }, [inputUsername, inputPassword1, inputPassword2]);

    useEffect(() => {
        let valid = true;
        for (let errorList of Object.values(errorSchema)) {
            errorList?.length > 0 && (valid = false);
        }

        setIsValid(valid);
    }, [errorSchema]);

    return (
        <div className="RegisterForm has-validation">
            {(serverErrors || []).map((error) => (
                <div className="text-danger">{error?.message}</div>
            ))}

            <div class="mb-3 has-validation">
                <label class="form-label">Login</label>

                <input
                    className={`form-control ${errorSchema["login"].length > 0 && `is-invalid`}`}
                    id="username"
                    name="username"
                    placeholder="Username"
                    value={inputUsername}
                    onChange={(e) => setInputUsername(e.target.value)}
                />
                {(errorSchema["login"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>
            <div class="mb-3 has-validation">
                <label class="form-label">Password</label>
                <input
                    type="password"
                    className={`form-control ${errorSchema["password1"].length > 0 && `is-invalid`}`}
                    placeholder="Password"
                    value={inputPassword1}
                    id="password"
                    name="password"
                    onChange={(e) => setInputPassword1(e.target.value)}
                />
                {(errorSchema["password1"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <div class="mb-3 has-validation">
                <label class="form-label">Repeat password</label>
                <input
                    type="password"
                    className={`form-control ${errorSchema["password2"].length > 0 && `is-invalid`}`}
                    placeholder="Repeat password"
                    value={inputPassword2}
                    id="password2"
                    name="password2"
                    onChange={(e) => setInputPassword2(e.target.value)}
                />
                {(errorSchema["password2"] || []).map((error) => (
                    <div className="invalid-feedback ">{error}</div>
                ))}
            </div>

            <button
                disabled={!isValid}
                className="btn btn-success "
                onClick={() => onRegister(inputUsername, inputPassword1)}
            >
                Register
            </button>
        </div>
    );
};

export const CRegisterForm = connect((state) => ({ serverErrors: state.promise?.register?.error || [] }), {
    onRegister: (login, password) => actionRegister(login, password),
})(RegisterForm);
