import { Link } from "react-router-dom";
import { AuthPopUp } from "../components/AuthPopUp";
import { connect, useSelector } from "react-redux";
import CLogo from "../components/Logo";
import { CSearchBar, CSearchResults } from "../components/SearchBar";
import { useEffect, useState } from "react";
import CCartIcon from "../components/CartIcon";
import { CProfileImage } from "../components/ProfileImage";
import { AppBar, Box, Toolbar, Stack, Container, Button } from "@mui/material";
import { DrawerCart } from "components/DrawerCart";

export const Header = ({ children }) => {
    const token = useSelector((state) => state.auth?.token || null);

    const [isAuthPopupOpen, setIsAuthPopupOpen] = useState(false);
    const [isCartDrawerOpen, setIsCartDrawerOpen] = useState(false);

    return (
        <Box className="Header">
            <AppBar>
                <Container>
                    <Toolbar variant="dense">
                        <CLogo />

                        <Box sx={{ flexGrow: 1 }}>
                            <CSearchBar render={CSearchResults} renderParams={{ itemLink: "/good/" }} />
                        </Box>

                        {!token && (
                            <Button color="inherit" onClick={() => setIsAuthPopupOpen(true)}>
                                Войти
                            </Button>
                        )}

                        <Stack direction="row" spacing={2}>
                            <AuthPopUp isOpen={isAuthPopupOpen} onPopupClose={() => setIsAuthPopupOpen(false)} />
                            {token && (
                                <Link className="Link" to={`/profile/`}>
                                    <Box sx={{ maxHeight: 35, maxWidth: 35 }}>
                                        <CProfileImage />
                                    </Box>
                                </Link>
                            )}
                            {token && (
                                <Box onClick={() => setIsCartDrawerOpen(true)}>
                                    <CCartIcon />
                                </Box>
                            )}
                        </Stack>
                    </Toolbar>
                </Container>
            </AppBar>
            <DrawerCart isOpen={isCartDrawerOpen} onClose={() => setIsCartDrawerOpen(false)} />

            {children}
        </Box>
    );
};
