import { Redirect, Route, Switch } from "react-router-dom";
import { useEffect } from "react";
import { store } from "../reducers";
import { actionCatById, actionGoodById, actionGoodsFind, actionOrders } from "../actions";
import { connect, useDispatch } from "react-redux";
import CCategory, { GoodList } from "../pages/Category";
import CCart from "../pages/Cart";
import CDashboard from "../components/Dashboard";

import Aside from "./Aside";
import Content from "./Content";
import CGoodDetail, { GoodDetail } from "../pages/GoodDetail";
import { CProfilePage, ProfilePage } from "../pages/ProfilePage";
import { AdminPanel } from "../pages/admin";
import { CProtectedRoute, ProtectedRoute } from "../components/ProtectedRoute";
import { HomePage } from "../pages";
import ScrollToTop from "../components/ScrollToTop";
import { Container, Stack } from "@mui/material";

const CategoryContainer = ({
    match: {
        params: { _id },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionCatById(_id));
    }, [_id]);
    return <CCategory />;
};

const CGoodsList = connect((state) => ({ goods: state.promise?.pageGoodsFind?.payload || [] }))(GoodList);

const GoodsListContainer = ({
    match: {
        params: { searchData },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionGoodsFind({ text: searchData, promiseName: "pageGoodsFind" }));
    }, [searchData]);

    return <CGoodsList />;
};

const GoodContainer = ({
    match: {
        params: { _id },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionGoodById({ _id }));
    }, [_id]);
    return <CGoodDetail />;
};

const ProfilePageContainer = () => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionOrders());
    });
    return <CProfilePage />;
};

const Main = ({ children }) => (
    <Container className="Main">
        <Stack direction="row">
            <Aside />
            <Content>
                <ScrollToTop />
                <Switch>
                    <Route path="/category/:_id" exact component={CategoryContainer} />
                    <Route path="/good/:_id/" component={GoodContainer} exact />
                    <Route path="/search/:searchData/" component={GoodsListContainer} exact />
                    <Route path="/" component={HomePage} exact />
                    <CProtectedRoute path="/cart" roles={["admin", "user"]} component={CCart} />
                    <CProtectedRoute path="/profile" roles={["admin", "user"]} component={ProfilePageContainer} />
                    <CProtectedRoute path="/admin" roles={["admin"]} component={AdminPanel} />

                    {/* <Redirect from ="/" to="/categories/" /> */}
                </Switch>
            </Content>
        </Stack>

        {children}
    </Container>
);

export default Main;
