import { useEffect, useState } from "react";
import { backendURL } from "helpers";
import CBuyButton from "components/BuyButton";
import { connect, useSelector } from "react-redux";
import { actionCartDelete } from "reducers";
import defaultGoodImage from "images/defaultGoodImage.png";
import { Divider, Grid, Paper, Stack, Typography } from "@mui/material";
import { Box } from "@mui/system";

export const GoodDetail = ({ good, cart }) => {
    const token = useSelector((state) => state.auth?.token || null);
    const { _id = "", name = "", price = "", description = "", images = [] } = good || {};
    const [inCart, setInCart] = useState(0);

    useEffect(() => {
        setInCart((cart[_id] && cart[_id].count) || 0);
    }, [_id, cart]);

    return (
        <Paper className="GoodDetail page">
            <Grid container>
                <Grid item xs={12} md={4}>
                    <Box
                        component="img"
                        src={images[0]?.url ? `${backendURL}/${images ? images[0]?.url : ""}` : defaultGoodImage}
                        sx={{ maxWidth: "100%", p: 3 }}
                    />
                </Grid>
                <Grid item xs={12} md={8}>
                    <Stack spacing={2} sx={{ p: 3 }}>
                        <Typography variant="h4">{name}</Typography>
                        <Divider />
                        <Typography variant="body1">
                            {" "}
                            <b>Цена:</b> {price}
                        </Typography>
                        <Typography variant="body1" sx={{ minHeight: 100 }}>
                            <b>Описание:</b>
                            <br />
                            {description}
                        </Typography>

                        {token ? <CBuyButton good={{ name, images, price, _id }} /> : ""}
                    </Stack>
                </Grid>
            </Grid>
        </Paper>
    );
};

const CGoodDetail = connect((state) => ({
    good: state.promise.goodById?.payload || {},
    cart: state.cart || {},
}))(GoodDetail);

export default CGoodDetail;
