import { AdminCategoryItem, AdminCategoryListHeader } from ".";
import { connect } from "react-redux";
import { useEffect, useState } from "react";
import { actionCatsFind } from "actions/actionCatsFind";
import { actionPromiseClear } from "reducers";
import { SearchBar } from "components/SearchBar";
import { SearchResults } from "components/SearchBar";
import { Box, Table, TableBody, TableHead } from "@mui/material";

const CSearchBar = connect(null, {
    onSearch: (text) => actionCatsFind({ promiseName: "adminCatsFind", text, limit: 5 }),
    onSearchButtonClick: () => actionPromiseClear("adminCatsFind"),
})(SearchBar);

const CSearchResults = connect((state) => ({ items: state.promise.adminCatsFind?.payload || [] }))(SearchResults);

const sortOptions = {
    true: {
        _id: (a, b) => (a["_id"] < b["_id"] ? 1 : -1),
        name: (a, b) => (a["name"] < b["name"] ? 1 : -1),
        parent: (a, b) => (a?.parent?.name < b?.parent?.name ? 1 : -1),
    },
    false: {
        _id: (a, b) => (a["_id"] > b["_id"] ? 1 : -1),
        name: (a, b) => (a["name"] > b["name"] ? 1 : -1),
        parent: (a, b) => (a?.parent?.name > b?.parent?.name ? 1 : -1),
    },
};

const AdminCategoryList = ({ categories }) => {
    const [sortBy, setSortBy] = useState("_id");
    const [isSortReverse, setSortReverse] = useState(false);

    useEffect(() => {
        setSortReverse(false);
    }, [sortBy]);

    return (
        <Box className="AdminCategoryList">
            <Box className="searchBarWrapper">
                <CSearchBar
                    render={CSearchResults}
                    searchLink="/admin/category/search/"
                    renderParams={{ itemLink: "/admin/category/" }}
                />
            </Box>
            <Table>
                <TableHead>
                    <AdminCategoryListHeader
                        sortReversed={isSortReverse}
                        onSortReverseChange={() => setSortReverse((isSortReverse) => !isSortReverse)}
                        sort={sortBy}
                        onSortChange={(sortBy) => setSortBy(sortBy)}
                    />
                </TableHead>
                <TableBody>
                    {(categories || []).sort(sortOptions[isSortReverse][sortBy]).map((cat) => (
                        <AdminCategoryItem category={cat} key={cat._id} />
                    ))}
                </TableBody>
            </Table>
        </Box>
    );
};

const CAdminCategoryList = connect((state) => ({ categories: state.feed?.payload || [] }))(AdminCategoryList);

export { AdminCategoryList, CAdminCategoryList };
