import { AdminGoodItem, AdminGoodListHeader } from ".";
import { connect } from "react-redux";
import { useEffect, useState } from "react";

import { SearchBar, SearchResults } from "components/SearchBar";
import { actionGoodsFind } from "actions";
import { actionPromiseClear } from "reducers";
import { Box, Table, TableBody, TableHead } from "@mui/material";

const CSearchBar = connect(null, {
    onSearch: (text) => actionGoodsFind({ promiseName: "adminGoodsFind", text, limit: 5 }),
    onSearchButtonClick: () => actionPromiseClear("adminGoodsFind"),
})(SearchBar);

const CSearchResults = connect((state) => ({ items: state.promise.adminGoodsFind?.payload || [] }))(SearchResults);

const sortOptions = {
    true: {
        _id: (a, b) => (a["_id"] < b["_id"] ? 1 : -1),
        name: (a, b) => (a["name"] < b["name"] ? 1 : -1),
        price: (a, b) => (a["price"] < b["price"] ? 1 : -1),
    },
    false: {
        _id: (a, b) => (a["_id"] > b["_id"] ? 1 : -1),
        name: (a, b) => (a["name"] > b["name"] ? 1 : -1),
        price: (a, b) => (a["price"] > b["price"] ? 1 : -1),
    },
};

const AdminGoodList = ({ goods }) => {
    const [sortBy, setSortBy] = useState("_id");
    const [isSortReverse, setSortReverse] = useState(false);

    useEffect(() => {
        setSortReverse(false);
    }, [sortBy]);

    useEffect(() => console.log(goods.reverse()), [isSortReverse]);
    return (
        <Box className="AdminGoodList">
            <Box className="searchBarWrapper">
                <CSearchBar
                    render={CSearchResults}
                    searchLink="/admin/goods/search/"
                    renderParams={{ itemLink: "/admin/good/" }}
                />
            </Box>
            <Table>
                <TableHead>
                    <AdminGoodListHeader
                        sortReversed={isSortReverse}
                        onSortReverseChange={() => setSortReverse((isSortReverse) => !isSortReverse)}
                        sort={sortBy}
                        onSortChange={(sortBy) => setSortBy(sortBy)}
                    />
                </TableHead>
                <TableBody>
                    {(goods || []).sort(sortOptions[isSortReverse][sortBy]).map((good) => (
                        <AdminGoodItem good={good} key={good._id} />
                    ))}
                </TableBody>
            </Table>
        </Box>
    );
};

const CAdminGoodList = connect((state) => ({ goods: state.feed?.payload || [] }))(AdminGoodList);

export { AdminGoodList, CAdminGoodList };
