import { Route, Switch } from "react-router-dom";
import { CAdminCategoryList } from "./AdminCategory";
import { CCategoryTree } from "./CategoryTree";
import { useEffect } from "react";
import { connect, useDispatch, useSelector } from "react-redux";
import { actionFeedAdd, actionFeedCats, actionFeedClear, actionFeedGoods, actionPromiseClear, store } from "reducers";
import { actionCatById, actionGoodById } from "actions";
import { CAdminCategoryDetail } from "./AdminCategory/AdminCategoryDetail";
import { CAdminGoodDetail, CAdminGoodList } from "./AdminGoods";
import { actionFeedGoodsFind, actionFeedCatsFind } from "../../reducers";
import { Paper } from "@mui/material";

const AdminCategoryListContainer = ({ cats }) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionFeedCats(cats?.length || 0));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedCatAll },
                } = store.getState();
                if (feedCatAll.status !== "PENDING") {
                    dispatch(actionFeedCats(feed.payload?.length || 0));
                }
            }
        };
        return () => {
            dispatch(actionFeedClear());
            dispatch(actionPromiseClear("feedCatAll"));

            dispatch(actionPromiseClear("categoryUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (cats.length) dispatch(actionFeedAdd(cats));
    }, [cats]);
    return <CAdminCategoryList />;
};

const AdminGoodListContainer = ({ goods }) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionFeedGoods());
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedGoodsAll },
                } = store.getState();
                if (feedGoodsAll.status !== "PENDING") {
                    dispatch(actionFeedGoods(feed.payload?.length || 0));
                }
            }
        };
        return () => {
            dispatch(actionFeedClear());
            dispatch(actionPromiseClear("feedGoodsAll"));
            dispatch(actionPromiseClear("goodUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (goods.length) store.dispatch(actionFeedAdd(goods));
    }, [goods]);
    return <CAdminGoodList />;
};

const AdminGoodListSearchContainer = ({
    goods,
    match: {
        params: { searchData },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionFeedClear());
        dispatch(actionFeedGoodsFind({ text: searchData }));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedGoodsFind },
                } = store.getState();
                if (feedGoodsFind.status !== "PENDING") {
                    dispatch(actionFeedGoodsFind({ skip: feed.payload?.length || 0, text: searchData }));
                }
            }
        };
        return () => {
            dispatch(actionFeedClear());
            dispatch(actionPromiseClear("feedGoodsFind"));
            dispatch(actionPromiseClear("goodUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (goods.length) dispatch(actionFeedAdd(goods));
    }, [goods]);
    return <CAdminGoodList />;
};

const AdminCategoryListSearchContainer = ({
    cats,
    match: {
        params: { searchData },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(actionFeedClear());
        dispatch(actionFeedCatsFind({ text: searchData }));
        window.onscroll = (e) => {
            if (window.innerHeight + window.scrollY >= document.body.offsetHeight) {
                const {
                    feed,
                    promise: { feedCatsFind },
                } = store.getState();
                if (feedCatsFind.status !== "PENDING") {
                    dispatch(actionFeedCatsFind({ skip: feed.payload?.length || 0, text: searchData }));
                }
            }
        };
        return () => {
            dispatch(actionFeedClear());
            dispatch(actionPromiseClear("feedCatsFind"));
            dispatch(actionPromiseClear("categoryUpsert"));
            window.onscroll = null;
        };
    }, []);

    useEffect(() => {
        if (cats.length) store.dispatch(actionFeedAdd(cats));
    }, [cats]);
    return <CAdminCategoryList />;
};

const AdminCategoryDetailContainer = ({
    match: {
        params: { _id },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        if (_id) {
            dispatch(actionCatById(_id, "adminCatById"));
        } else {
            dispatch(actionPromiseClear("adminCatById"));
        }
    }, [_id]);
    return <CAdminCategoryDetail />;
};

const AdminGoodDetailContainer = ({
    match: {
        params: { _id },
    },
}) => {
    const dispatch = useDispatch();
    useEffect(() => {
        if (_id) {
            dispatch(actionGoodById({ _id, promiseName: "adminGoodById" }));
        } else {
            dispatch(actionGoodById("adminGoodById"));
        }
    }, [_id]);
    return <CAdminGoodDetail />;
};

const CAdminCategoryListContainer = connect((state) => ({ cats: state.promise?.feedCatAll?.payload || [] }))(
    AdminCategoryListContainer
);

const CAdminGoodListContainer = connect((state) => ({ goods: state.promise?.feedGoodsAll?.payload || [] }))(
    AdminGoodListContainer
);

const CAdminGoodListSearchContainer = connect((state) => ({ goods: state.promise?.feedGoodsFind?.payload || [] }))(
    AdminGoodListSearchContainer
);

const CAdminCategoryListSearchContainer = connect((state) => ({ cats: state.promise?.feedCatsFind?.payload || [] }))(
    AdminCategoryListSearchContainer
);

export const AdminPanel = () => {
    return (
        <Paper className="AdminPanel page">
            <Switch>
                <Route path="/admin/" component={CCategoryTree} exact />
                <Route path="/admin/categories/" component={CAdminCategoryListContainer} exact />
                <Route path="/admin/goods/" component={CAdminGoodListContainer} exact />
                <Route path="/admin/goods/search/:searchData/" component={CAdminGoodListSearchContainer} exact />
                <Route path="/admin/category/search/:searchData/" component={CAdminCategoryListSearchContainer} exact />
                <Route path="/admin/category/:_id?" component={AdminCategoryDetailContainer} />
                <Route path="/admin/good/:_id?" component={AdminGoodDetailContainer} exact />
            </Switch>
        </Paper>
    );
};
