import "react-sortable-tree/style.css";
import { Component, useState, useEffect } from "react";
import { connect } from "react-redux";
import SortableTree from "react-sortable-tree";
import { FaEdit } from "react-icons/fa";
import { actionCategoryUpsert } from "actions";
import { CategoryEditPopUp } from "../CategoryEditPopUp";
import { Button } from "@mui/material";

const bulidCategoryTree = (list) => {
    let node,
        roots = [],
        map = {},
        newList = [];

    for (let i = 0; i < list.length; i += 1) {
        newList[i] = {};
        newList[i].children = [];
        newList[i].title = list[i].name;
        newList[i].parent = list[i].parent;
        newList[i]._id = list[i]._id;
        map[newList[i]._id] = i;
    }

    for (let i = 0; i < list.length; i += 1) {
        node = newList[i];
        if (list[i].subCategories) {
            for (let subCat of list[i].subCategories) {
                node.children.push(newList[map[subCat._id]]);
            }
        }
    }

    for (let i = 0; i < list.length; i += 1) {
        node = newList[i];
        if (node.parent === null) {
            roots.push(node);
        }
    }
    return roots;
};

const bulidCategoryList = (tree) => {
    let list = [];
    for (let node of tree) {
        list = [...list, node];
        if (!!node.children?.length) {
            list = [...list, ...bulidCategoryList(node.children)];
        }
    }
    return list;
};

export const CategoryTree = ({ categories, onDrop, onPopupOpen }) => {
    const [treeData, setTreeData] = useState([]);
    const [selectedNode, setSelectedNode] = useState(null);
    const [isCategoryPopupOpen, setIsCategoryPopupOpen] = useState(false);

    useEffect(() => {
        setTreeData(bulidCategoryTree(categories));
    }, [categories]);

    return (
        <>
            <div className="CategotyTree">
                <CategoryEditPopUp
                    category={selectedNode}
                    isOpen={isCategoryPopupOpen}
                    onClose={() => setIsCategoryPopupOpen(false)}
                />
                <SortableTree
                    isVirtualized={false}
                    treeData={treeData}
                    onChange={(treeData) => setTreeData(treeData)}
                    generateNodeProps={({ node, parentNode }) => ({
                        onDrop: () => {
                            const { _id, title: name } = node;
                            if (parentNode) {
                                let { _id, title: name } = parentNode;
                                let subCategories =
                                    parentNode?.children?.map(({ _id, title: name }) => ({ _id, name })) || [];
                                onDrop({ _id, name, subCategories });
                            } else {
                                onDrop({ _id, name, parent: null });
                            }
                        },
                        buttons: [
                            <Button
                                className="editButton"
                                onClick={() => {
                                    let parent = null;
                                    let { title: name, children: subCategories, _id } = { ...node };
                                    subCategories = subCategories.map(({ _id, title: name }) => ({ _id, name }));
                                    if (parentNode) {
                                        let { title: name, _id } = parentNode;
                                        parent = { _id, name };
                                    }
                                    setSelectedNode({ name, _id, subCategories, parent });
                                    setIsCategoryPopupOpen(true);
                                }}
                            >
                                <FaEdit />
                            </Button>,
                        ],
                        className: "TreeNode",
                    })}
                />
            </div>
        </>
    );
};

export const CCategoryTree = connect((state) => ({ categories: state.promise?.catAll?.payload || [] }), {
    onDrop: (category) => actionCategoryUpsert(category),
})(CategoryTree);
