import { call, put, takeEvery, takeLatest } from "redux-saga/effects";
import { actionPromise } from ".";
import { actionCatAll, actionGoodsFind } from "../actions";
import { actionCatsFind } from "../actions/actionCatsFind";
import { actionGoodsAll } from "../actions/actionGoodsAll";
import { gql } from "../helpers";
import { promiseWorker } from "./promiseReducer";

function feedReducer(state = { payload: [] }, { type, payload = [] }) {
    if (type === "FEED_ADD") {
        return {
            ...state,
            payload: [...state["payload"], ...payload],
        };
    }

    if (type === "FEED_CLEAR") {
        return { payload: [] };
    }
    return state || { payload: [] };
}

const actionFeedAdd = (payload) => ({ type: "FEED_ADD", payload });
const actionFeedClear = () => ({ type: "FEED_CLEAR" });

const actionFeedGoods = (skip = 0) => ({ type: "FEED_GOODS", payload: { skip } });
function* feedGoodsWorker(action) {
    const { skip } = action.payload;
    yield put(actionGoodsAll({ skip, limit: 50, promiseName: "feedGoodsAll" }));
}

const actionFeedGoodsFind = ({ skip = 0, text = "" }) => ({ type: "FEED_GOODS_FIND", payload: { skip, text } });
function* feedGoodsFindWorker(action) {
    const { skip, text } = action.payload;
    yield put(actionGoodsFind({ skip, limit: 50, promiseName: "feedGoodsFind", text }));
}

const actionFeedCatsFind = ({ skip = 0, text = "" }) => ({ type: "FEED_CATS_FIND", payload: { skip, text } });
function* feedCatsFindWorker(action) {
    const { skip, text } = action.payload;
    yield put(actionCatsFind({ skip, promiseName: "feedCatsFind", text, limit: 50 }));
}

const actionFeedCats = (skip = 0) => ({ type: "FEED_CATS", payload: { skip } });
function* feedCatsWorker(action) {
    const { skip } = action.payload;
    yield put(actionCatAll({ promiseName: "feedCatAll", skip, limit: 50 }));
}

export {
    actionFeedCats,
    actionFeedCatsFind,
    actionFeedGoods,
    actionFeedClear,
    actionFeedAdd,
    actionFeedGoodsFind,
    feedReducer,
};

export function* feedWatcher() {
    yield takeEvery("FEED_GOODS", feedGoodsWorker);
    yield takeEvery("FEED_GOODS_FIND", feedGoodsFindWorker);
    yield takeEvery("FEED_CATS_FIND", feedCatsFindWorker);
    yield takeEvery("FEED_CATS", feedCatsWorker);
}
