import { createStore, combineReducers, applyMiddleware } from "redux";
import thunk from "redux-thunk";
import createSagaMiddleware from "redux-saga";
import { all, put, takeEvery, takeLatest, takeLeading, select } from "redux-saga/effects";
import { authReducer, actionAuthLogin, actionAuthLogout } from "./authReducer";
import { composeWithDevTools } from "@redux-devtools/extension";
import {
    promiseReducer,
    actionPending,
    actionFulfilled,
    actionRejected,
    actionPromise,
    actionPromiseClear,
    promiseWatcher,
} from "./promiseReducer";
import { cartReducer, actionCartAdd, actionCartChange, actionCartDelete, actionCartClear } from "./cartReducer";
import {
    actionFeedCats,
    actionFeedCatsFind,
    actionFeedGoods,
    actionFeedGoodsFind,
    actionFeedClear,
    actionFeedAdd,
    feedReducer,
    feedWatcher,
} from "./feedReducer";

export { cartReducer, actionCartAdd, actionCartChange, actionCartDelete, actionCartClear };
export { authReducer, actionAuthLogin, actionAuthLogout };
export { promiseReducer, actionPending, actionFulfilled, actionRejected, actionPromise, actionPromiseClear };
export {
    actionFeedCats,
    actionFeedCatsFind,
    actionFeedGoods,
    actionFeedGoodsFind,
    actionFeedClear,
    actionFeedAdd,
    feedReducer,
};

export const sagaMiddleware = createSagaMiddleware();

export const store = createStore(
    combineReducers({
        auth: authReducer,
        promise: promiseReducer,
        cart: cartReducer,
        feed: feedReducer,
    }),
    composeWithDevTools(applyMiddleware(sagaMiddleware))
);

export function* rootSaga() {
    yield all([
        promiseWatcher(),
        feedWatcher(),
        // loginWatcher()
    ]);
}

sagaMiddleware.run(rootSaga);
