import { aboutMeWorker } from "actions/actionAboutMe";
import { catAllWorker } from "actions/actionCatAll";
import { catByIdWorker } from "actions/actionCatById";
import { categoryUpdateWorker } from "actions/actionCategoryUpdate";
import { categoryUpsertWorker } from "actions/actionCategoryUpsert";
import { catsFindWorker } from "actions/actionCatsFind";
import { goodByIdWorker } from "actions/actionGoodById";
import { goodsAllWorker } from "actions/actionGoodsAll";
import { goodsFindWorker } from "actions/actionGoodsFind";
import { goodUpdateWorker } from "actions/actionGoodUpdate";
import { loginWorker } from "actions/actionLogin";
import { logoutWorker } from "actions/actionLogout";
import { newOrderWorker } from "actions/actionNewOrder";
import { ordersWorker } from "actions/actionOrders";
import { pageStartWorker } from "actions/actionPageStart";
import { registerWorker } from "actions/actionRegister";
import { rootCatsWorker } from "actions/actionRootCats";
import { updateAvatarWorker } from "actions/actionUpdateAvatar";
import { uploadFilesWorker } from "actions/actionUploadFiles";
import { userUpsertWorker } from "actions/actionUserUpsert";
import { put, takeEvery, takeLatest } from "redux-saga/effects";

export function promiseReducer(state = {}, { type, name, status, payload, error }) {
    if (type === "PROMISE") {
        return {
            ...state,
            [name]: { status, payload: status === "PROMISE" ? [name].payload : payload, error },
        };
    }
    if (type === "PROMISE_CLEAR") {
        const { [name]: toRemove, ...newState } = state;
        return newState;
    }
    return state;
}

export const actionPending = (name) => ({ type: "PROMISE", name, status: "PENDING" });
export const actionFulfilled = (name, payload) => ({ type: "PROMISE", name, status: "FULFILLED", payload });
export const actionRejected = (name, error) => ({ type: "PROMISE", name, status: "REJECTED", error });
export const actionPromiseClear = (name) => ({ type: "PROMISE_CLEAR", name });
export const actionPromise = (name, promise) => ({ type: "PROMISE_START", name, promise });

export function* promiseWorker(action) {
    const { name, promise } = action;
    yield put(actionPending(name));
    try {
        let data = yield promise;
        yield put(actionFulfilled(name, data));
        return data;
    } catch (error) {
        yield put(actionRejected(name, error));
    }
}

export function* promiseWatcher() {
    yield takeEvery("PROMISE_START", promiseWorker);
    yield takeEvery("PAGE_START", pageStartWorker);
    yield takeLatest("CATEGORY_UPDATE", categoryUpdateWorker);
    yield takeLatest("USER_UPSERT", userUpsertWorker);
    yield takeLatest("GOOD_UPDATE", goodUpdateWorker);
    yield takeLatest("LOGIN_FULL", loginWorker);
    yield takeLatest("NEW_ORDER", newOrderWorker);
    yield takeLatest("REGISTER_FULL", registerWorker);
    yield takeLatest("UPDATE_AVATAR", updateAvatarWorker);
    yield takeLatest("USER_UPDATE", updateAvatarWorker);
    yield takeEvery("UPLOAD_FILES", uploadFilesWorker);
    yield takeLatest("ORDERS", ordersWorker);
    yield takeLatest("LOGOUT_FULL", logoutWorker);
    yield takeLatest("ABOUT_ME", aboutMeWorker);
    yield takeLatest("CAT_ALL", catAllWorker);
    yield takeLatest("CAT_BY_ID", catByIdWorker);
    yield takeLatest("GOOD_BY_ID", goodByIdWorker);
    yield takeLatest("ROOT_CATS", rootCatsWorker);
    yield takeLatest("CATS_FIND", catsFindWorker);
    yield takeLatest("GOODS_ALL", goodsAllWorker);
    yield takeLatest("GOODS_FIND", goodsFindWorker);
    yield takeLatest("CATEGORY_UPSERT", categoryUpsertWorker);
}
